%
% DoDCT44QP
%
% compute MSE & PSNR as fcns of QP for 4x4 Matlab DCT2
%
% THIS VERSION USED FOR ICIP 2021 try
%
% As in Iain 2ed, use round (see Iain 2ed p. 198)
%
%
clear;
%--- FOREMAN -------------------------
%InFileR = 'foreR44';
%OutFileMSE = 'foreQP_MSE_R_dct44';
%OutFilePSNR = 'foreQP_PSNR_R_dct44';
%--- NEWS ----------------------------
%InFileR = 'newsR44';
%OutFileMSE = 'newsQP_MSE_R_dct44';
%OutFilePSNR = 'newsQP_PSNR_R_dct44';
%--- STEFAN --------------------------
%InFileR = 'stefanR44';
%OutFileMSE = 'stefanQP_MSE_R_dct44';
%OutFilePSNR = 'stefanQP_PSNR_R_dct44';
%--- TEMPETE -------------------------
InFileR = 'tempeteR44';
OutFileMSE = 'tempeteQP_MSE_R_dct44';
OutFilePSNR = 'tempeteQP_PSNR_R_dct44';




load(InFileR,'R');
[nRows nCols nFrame] = size(R);


%---------------------------------------------------
% RUN EXPERIMENTS for 0 \leq QP \leq 51
%---------------------------------------------------

% array of QP values and base Qstep array for 0 \leq QP \leq 5
%   For QstepBase, see Iain 2ed p. 202, 
QParray = 0:51;
QstepBase = [0.625 0.6875 0.8125 0.875 1.0 1.125];

% Cummulative MSE over frames calculated at each value of QP / Qstep
MSE_R = zeros(1,52);

% loop on frames
for iFrame = 1:nFrame
  iFrame   % echo frame counter to console
  Rframe = R(:,:,iFrame);

  % Perform 4x4 block DCT
  X = blkproc(Rframe,[4 4],@dct2);

  % loop on QP w/ loop counter k
  for k = 1:52
    QP = QParray(k);
    Qstep = QstepBase(mod(QP,6)+1) * 2^floor(QP/6);

    % Quantize as in Iain 2ed p. 198
    XQ = round(X/Qstep);

    % Invert
    Rframe2 = round(blkproc(Qstep*XQ,[4 4],@idct2));
    MSE_R(k) = MSE_R(k) + mean2((Rframe2-Rframe).^2);
  end         % for k
end           % for iFrame

% Convert MSE to average over all frames
MSE_R = MSE_R / nFrame;

% Compute PSNR
PSNR_R = 20*log10(255*255) - 10*log10(MSE_R);

% Save
save(OutFileMSE,'MSE_R');
save(OutFilePSNR,'PSNR_R');
