%
% Compute MSE and PSNR as a function of QP for H2.264 4x4 transform
%
%   As in Iain 2ed, use round (see Iain 2ed p. 198)
%
%   THIS VERSION USED FOR ICIP 2021 try
%
clear;
%--- FOREMAN -------------------------
%InFileR = 'foreR44';
%OutFileMSE = 'foreQP_MSE_R_h264';
%OutFilePSNR = 'foreQP_PSNR_R_h264';
%--- NEWS ----------------------------
%InFileR = 'newsR44';
%OutFileMSE = 'newsQP_MSE_R_h264';
%OutFilePSNR = 'newsQP_PSNR_R_h264';
%--- STEFAN --------------------------
%InFileR = 'stefanR44';
%OutFileMSE = 'stefanQP_MSE_R_h264';
%OutFilePSNR = 'stefanQP_PSNR_R_h264';
%--- TEMPETE -------------------------
InFileR = 'tempeteR44';
OutFileMSE = 'tempeteQP_MSE_R_h264';
OutFilePSNR = 'tempeteQP_PSNR_R_h264';

load(InFileR,'R');
[nRows nCols nFrame] = size(R);

%---------------------------------------------------
% RUN EXPERIMENTS for 0 \leq QP \leq 51
%---------------------------------------------------

% Set up transform matrices that do not depend on QP
[Cf4, Ci4, Sf4, Si4, Vi4] = H2644x4Setup;


% array of QP values and base Qstep array for 0 \leq QP \leq 5
%   For QstepBase, see Iain 2ed p. 202, 
QParray = 0:51;
QstepBase = [0.625 0.6875 0.8125 0.875 1.0 1.125];

% Cummulative MSE over frames calculated at each value of nbits
MSE_R = zeros(1,52);

% loop on frames
for iFrame = 1:nFrame
  iFrame
  Rframe = R(:,:,iFrame);

  % Unlike matlab DCT and HOT, the transform can NOT be performed here.
  %   The reason is that the Vi and Mf matrices of the forward transform
  %   and the Vi matrix of the reverse transform depend on QP.
  %     QP in turn depends on the bit depth for coefs.
  %     >>> so the transform must be called within the QP loop on k.

  % loop on QP w/ loop counter k
  for k = 1:52
    QP = QParray(k);
    Qstep = QstepBase(mod(QP,6)+1) * 2^floor(QP/6);

    % Do the forward transform. Quantization is built in.
    X = blkproc(Rframe,[4 4],@H2644x4,QP,Vi4,Si4,Sf4,Cf4);

    % Invert
    Rframe2 = blkproc(X,[4 4],@iH2644x4,QP,Vi4,Ci4);

    MSE_R(k) = MSE_R(k) + mean2((Rframe2-Rframe).^2);
  end;         % for k
end;           % for iFrame

% Convert MSE to average over all frames
MSE_R = MSE_R / nFrame;

% Compute PSNR
PSNR_R = 20*log10(255*255) - 10*log10(MSE_R);

save(OutFileMSE,'MSE_R');
save(OutFilePSNR,'PSNR_R');
