function [Cf4, Ci4, Sf4, Si4, Vi4] = H2644x4Setup
%
% Set up matrices used by forward and inverse H.264
% approximate DCT 4x4 orthogonal intenger transforms.
%
% These are the matrices that do not depend on QP.
%
% See Iain Richardson, 2ed, Chapter 7.
%

Cf4 = [1 1 1 1;...
       2 1 -1 -2;... 
       1 -1 -1 1;... 
       1 -2 2 -1];

Sf4 = [1/4 1/(2*sqrt(10)) 1/4 1/(2*sqrt(10));... 
       1/(2*sqrt(10)) 1/10 1/(2*sqrt(10)) 1/10;... 
       1/4 1/(2*sqrt(10)) 1/4 1/(2*sqrt(10));... 
       1/(2*sqrt(10)) 1/10 1/(2*sqrt(10)) 1/10];

%4-point integer transform Basis Cf4
Ci4 = [1 1 1 1;...
       1 1/2 -1/2 -1;...
       1 -1 -1 1;...
       1/2 -1 1 -1/2];

%Scaling factor Si4 = Ri4.*Ri4T
Si4 = [1/4 1/sqrt(10) 1/4 1/sqrt(10);...
       1/sqrt(10) 2/5 1/sqrt(10) 2/5;...
       1/4 1/sqrt(10) 1/4 1/sqrt(10);...
       1/sqrt(10) 2/5 1/sqrt(10) 2/5];

%Vi8 for Q = 0 to 5
Vi40 = [10 13 10 13;...
        13 16 13 16;...
        10 13 10 13;...
        13 16 13 16];
    
Vi41 = [11 14 11 14;...
        14 18 14 18;...
        11 14 11 14;...
        14 18 14 18];
    
Vi42 = [13 16 13 16;...
        16 20 16 20;...
        13 16 13 16;...
        16 20 16 20];
    
    
Vi43 = [14 18 14 18;...
        18 23 18 23;...
        14 18 14 18;...
        18 23 18 23];
    
Vi44 = [16 20 16 20;...
        20 25 20 25;...
        16 20 16 20;...
        20 25 20 25];
    
Vi45 = [18 23 18 23;...
        23 29 23 29;...
        18 23 18 23;...
        23 29 23 29];

%make array of Q = 0 to 5 matrices
Vi4(:,:,1) = Vi40;
Vi4(:,:,2) = Vi41;
Vi4(:,:,3) = Vi42;
Vi4(:,:,4) = Vi43;
Vi4(:,:,5) = Vi44;
Vi4(:,:,6) = Vi45;

end
