%
% MakeResiduals44
%
% Read the sequence
% Get the Y component
% Do ME/MC from previous frame.  Repeat the FIRST frame to keep No. frames same as orig
%   pixel-level motion compensation from previous frame, 3x3 search range
% Write residuals to result ".mat" file as double
%
%

%
% Setup to choose video sequence
%
clear;
%--- FOREMAN -------------------------
%videoSequence = 'foreman_cif.yuv';
%width  = 352;
%height = 288;
%nFrame = 300;
%
%OutFileR = 'foreR44';
%--- NEWS ----------------------------
%videoSequence = 'news_cif.yuv';
%width  = 352;
%height = 288;
%nFrame = 300;
%
%OutFileR = 'newsR44';
%--- STEFAN --------------------------
%videoSequence = 'stefan_cif.yuv';
%width  = 352;
%height = 288;
%nFrame = 90;
%
%OutFileR = 'stefanR44';
%--- TEMPETE -------------------------
videoSequence = 'tempete_cif.yuv';
width  = 352;
height = 288;
nFrame = 260;

OutFileR = 'tempeteR44';


%
% Read the video sequence, get the Y component
%
[Y,U,V] = yuvRead(videoSequence, width, height ,nFrame);

%
% Compute the residuals
%
Yf = double(Y);
R = zeros(height,width,nFrame);

% MC done from previous frame.  To keep simple and size same, first frame of
%    R will be repeated
%
% iFrame is "current frame" counter in Yf... i.e., the frame *being* predicted
% oFrame is frame counter in R, should = iFrame
% pFrame is the reference frame counter in Yf, should = iFrame-1
%
% rowb,colb: top left corner of current block in Yf, loops over iFrame
% prow,pcol: top left corner of pred block in pFrame, loops over search range
%
% Yblock is the "current block" that is being predicted
% Pblock is the prediction block from pFrame, search loops on prow,pcol
% R is the array of residual frames
% 
for iFrame = 2:nFrame
  oFrame = iFrame;
  pFrame = iFrame-1;
  oFrame   % echo current frame counter to console
  for rowb = 1:4:height-4+1
    for colb = 1:4:width-4+1
      Yblock = Yf(rowb:rowb+3,colb:colb+3,iFrame);
      Er = 16*255;
      %
      % find pred block from prev frame that gives min err
      %
      prow_start = max(rowb-3,1);
      prow_stop = min(rowb+3,height-4+1);
      for prow = prow_start:prow_stop
        pcol_start = max(colb-3,1);
        pcol_stop = min(colb+3,width-4+1);
        for pcol = pcol_start:pcol_stop
          Pblock = Yf(prow:prow+3,pcol:pcol+3,pFrame);
          Rblock = Yblock - Pblock;
          Er2 = sum(sum(abs(Rblock)));
          if (Er2 < Er)
            Er = Er2;
            R(rowb:rowb+3,colb:colb+3,oFrame) = Rblock;
          end % if (Er2 < Er)
        end % for pcol
      end % for prow
    end % for colb
  end % for rowb
end % for oFrame

% Repeat the first frame to keep Num frames same
R(:,:,1) = R(:,:,2);
save(OutFileR,'R');





